function PSIRpicker(MOD,dvv,snr_win)

% PSIRPICKER    Pick P and S phase using a "Predict, Search, and Invert" algorithm.
% Copyright (C) 2016, Zefeng Li
%
% PSIRpicker(MOD): User defines velocity model. If MOD=0, pick prediction is done with the initial velocity model. If MOD=1,2,3,..., pick prediction is done with the most updated velocity model.If MOD=-1, run with default velocity model.
% PSIRpicker(MOD,dvv): User defines velocity model and search window range (dvv). For example, to set a search window range corresponding to 10% velocity perturbation, let dvv=0.10. dvv<0.3 is recommended. Default is 0.15.
% PSIRpicker(MOD,dvv,snr_win): User defines velocity model, search window range, sliding window lengths to calculate SNR function. snr_win is a two-element vector: snr_win=[p_snr_win, s_snr_win]. p_snr_win is for P and s_snr_win is for S. Typically, s_snr_win should be greater than p_snr_win, because S waves usually have lower SNR. 
% PSIRpicker: Run phase picking using default values: velocity model in southern California, 15% velocity perturbation to set search window range, and 0.2-s, 0.4-s sliding windows to calculate SNR function for P and S phases.
% 
% This script is written by Zefeng Li at Georgia Tech, Fri May 15 16:07:21 EDT 2015
%
% Reference:
% Li, Z. and Z. Peng, 2016. An automatic phase picker for local earthquakes with known locations: Combining a signal-to-noise ratio detector with 1D velocity model inversion, Seismo. Res. Lett..


% ----------------running mode-----------------------------------
if nargin<1
    vp = [5.95 0; 6.06 5; 6.23 10; 6.30 15; 6.37 20; 6.66 25; 7.74 30];
    vs = [3.41 0; 3.56 5; 3.70 10; 3.71 15; 3.54 20; 3.79 25; 4.52 30];
elseif(MOD==-1)
    vp = [5.95 0; 6.06 5; 6.23 10; 6.30 15; 6.37 20; 6.66 25; 7.74 30];
    vs = [3.41 0; 3.56 5; 3.70 10; 3.71 15; 3.54 20; 3.79 25; 4.52 30];
elseif(MOD==0)
    vp = load('vp0.dat');
    vs = load('vs0.dat');
elseif(MOD>0)
    vp = load('vp.dat');
    vs = load('vs.dat');
end

if nargin<2
    dvv = 0.15;
end

if nargin<3
    p_snr_win = 0.2;
    s_snr_win = 0.4;
else
    p_snr_win = snr_win(1);
    s_snr_win = snr_win(2);
end 

% ---------------initialization -----------------------------------

fid_cat = fopen('event.cat','r');
fid_pick = fopen('pick.dat','w');
fid_p = fopen('ptime.dat','w');
fid_s = fopen('stime.dat','w');

nvp = length(vp(:,1));
nvs = length(vs(:,1));

tline = fgets(fid_cat);
while ischar(tline)

%-------------------read data and remove mean-------------------------

    % read data
    tline_cell = strsplit(tline);
    station = char(tline_cell(1));
    event = char(tline_cell(2));
    filepath_e = char(tline_cell(3));
    filepath_n = char(tline_cell(4));
    filepath_z = char(tline_cell(5));
    file_e = sprintf('%s',filepath_e);
    file_n = sprintf('%s',filepath_n);
    file_z = sprintf('%s',filepath_z);
    fprintf('%s %s\n',station,event)

    if(exist(file_e,'file') & exist(file_n,'file') & exist(file_z,'file'))
    [te,UE,Ehdr] = fget_sac(file_e);
    [tn,UN,Nhdr] = fget_sac(file_n);
    [tz,UZ,Zhdr] = fget_sac(file_z);
    evlo = Zhdr.event.evlo;
    evla = Zhdr.event.evla;
    evdp = Zhdr.event.evdp;
    stlo = Zhdr.station.stlo;
    stla = Zhdr.station.stla;
    stel = Zhdr.station.stel;
    dist = Zhdr.evsta.dist; 
    t0 = Zhdr.times.o;
    dt = Zhdr.times.delta;
    evdp = evdp+stel/1000;
    evdp = max([evdp, 0.01]);

    % remove mean
    UE = UE-mean(UE);
    UN = UN-mean(UN);
    UZ = UZ-mean(UZ);

%--------------predict initial picks and decide search window--------------

    [ppick0 pdtt] = traveltime(evdp,dist,vp);
    [spick0 sdtt] = traveltime(evdp,dist,vs);
    p_search_win = [max([ppick0*dvv/(1+dvv),2*p_snr_win]), max([ppick0*dvv/(1-dvv),2*p_snr_win])];
    s_search_win = [max([spick0*dvv/(1+dvv),2*s_snr_win]), max([spick0*dvv/(1-dvv),2*s_snr_win])];

%---------------------- pick P,S -------------------------

    [ppick, p_snr] = picker(UZ,tz,t0,ppick0,p_search_win,p_snr_win);
    [sepick, se_snr] = picker(UE,te,max([t0,ppick]),spick0,s_search_win,s_snr_win);
    [snpick, sn_snr] = picker(UN,tn,max([t0,ppick]),spick0,s_search_win,s_snr_win);

%---------------------- recommend P S -------------------------


    if(se_snr>1 & sn_snr>1)
        spick = (sepick*se_snr+snpick*sn_snr)/(se_snr+sn_snr);
        s_snr = (se_snr+sn_snr)/2;
    elseif(se_snr>1)
        spick = sepick;
        s_snr = 0;
    elseif(sn_snr>1)
        spick = snpick;
        s_snr = 0;
    else
        spick = -1;
        s_snr = -5;
    end
        

%-------------------output picks and quality factors -------------------------

    fprintf(fid_pick,'%8s %s %8.3f %8.1f %8.3f %8.1f %8.3f %8.1f %8.3f %8.1f\n',station,event,ppick,p_snr,spick,s_snr,sepick,se_snr,snpick,sn_snr);

%-------------------output residual, quality factor, trave time in each layer-----------------
    if(p_snr>1)
        fprintf(fid_p,' %8.3f %8.3f ',ppick-ppick0,p_snr);
        fprintf(fid_p,' %8.5f ',pdtt);
        fprintf(fid_p,'\n');
    end
    
    if(s_snr>1)
        fprintf(fid_s,' %8.3f %8.3f ',spick-spick0,s_snr);
        fprintf(fid_s,' %8.5f ',sdtt);
        fprintf(fid_s,'\n');
    end

%-------------------clear variables and move to next case----------------------

    end

    clearvars -except fid_* dvv vp vs nvp nvs p_snr_win s_snr_win
    tline = fgets(fid_cat);

end

%------------------update velocity model-----------------------

ptime=load('ptime.dat');
prsl=ptime(:,1);
prslmax=max(abs(prsl));
hist(prsl,linspace(-prslmax,prslmax,9));
print(gcf,'-dpsc','hist_p.ps')

ptt=ptime(:,3:end);
ptt=ptt(:,any(ptt,1));
plambda = 10;

dvp = inv1d(prsl,ptt,plambda); 
dvp = [dvp;zeros(nvp-length(dvp),1)];
vp(:,1) = vp(:,1).*(1-dvp);
save('vp.dat','vp','-ascii');

stime=load('stime.dat');
srsl=stime(:,1);
srslmax=max(abs(srsl));
hist(srsl,linspace(-srslmax,srslmax,9));
print(gcf,'-dpsc','hist_s.ps')

stt=stime(:,3:end);
stt=stt(:,any(stt,1));
slambda = 10;

dvs = inv1d(srsl,stt,slambda); 
dvs = [dvs;zeros(nvs-length(dvs),1)];
vs(:,1) = vs(:,1).*(1-dvs);
save('vs.dat','vs','-ascii');

%--------------------------------------------------

fprintf('results written into pick.dat ptime.dat stime.dat\n');

fclose(fid_cat);
fclose(fid_pick);
fclose(fid_p);
fclose(fid_s);
